@@ -19,13 +19,14 @@ from .logistic import LogitNet
 
 def recipe_dropkick(
     adata,
-    X_final="raw_counts",
     filter=True,
+    min_genes=50,
     calc_metrics=True,
     mito_names="^mt-|^MT-",
     n_ambient=10,
     target_sum=None,
     n_hvgs=2000,
+    X_final="raw_counts",
     verbose=True,
 ):
     """
@@ -33,67 +34,71 @@ def recipe_dropkick(
 
     Parameters:
         adata (AnnData.AnnData): object with raw counts data in .X
-        X_final (str): which normalization should be left in .X slot?
-            ("raw_counts","arcsinh_norm","norm_counts")
-        filter (bool): remove cells and genes with zero total counts
+        filter (bool): remove cells with less than min_genes detected
+            and genes with zero total counts
+        min_genes (int): threshold for minimum genes detected. Default 50.
+            Ignored if filter==False.
         calc_metrics (bool): if False, do not calculate metrics in .obs/.var
         mito_names (str): substring encompassing mitochondrial gene names for
-            calculation of mito expression
+            calculation of mito expression. Ignored if calc_metrics==False.
         n_ambient (int): number of ambient genes to call. top genes by cells.
+            Ignored if calc_metrics==False.
         target_sum (int): total sum of counts for each cell prior to arcsinh 
-            and log1p transformations; default None to use median counts.
-        n_hvgs (int or None): number of HVGs to calculate using Seurat method
-            if None, do not calculate HVGs
+            or log1p transformations; default None to use median counts.
+        n_hvgs (int or None): number of HVGs to calculate using Seurat method.
+            if None, do not calculate HVGs.
+        X_final (str): which normalization layer should be left in .X slot?
+            ("raw_counts","arcsinh_norm","log1p_norm")
         verbose (bool): print updates to the console?
 
     Returns:
-        AnnData.AnnData: adata is edited in place to include:
-        - useful .obs and .var columns
-            ("total_counts", "pct_counts_mito", "n_genes_by_counts", etc.)
-        - raw counts (adata.layers["raw_counts"])
-        - normalized counts (adata.layers["norm_counts"])
-        - arcsinh transformation of normalized counts (adata.X)
-        - highly variable genes if desired (adata.var["highly_variable"])
+        adata (AnnData.AnnData): updated object includes:
+            - useful .obs and .var columns
+                ("total_counts", "pct_counts_mito", "n_genes_by_counts", etc.)
+            - raw counts (adata.layers["raw_counts"])
+            - arcsinh-transformed normalized counts (adata.layers["arcsinh_norm"])
+            - highly variable genes if desired (adata.var["highly_variable"])
     """
     if filter:
         # remove cells and genes with zero total counts
         orig_shape = adata.shape
-        sc.pp.filter_cells(adata, min_genes=10)
+        sc.pp.filter_cells(adata, min_genes=min_genes)
         sc.pp.filter_genes(adata, min_counts=1)
-        if adata.shape[0] != orig_shape[0]:
-            print(
-                "Ignoring {} cells with zero total counts".format(
-                    orig_shape[0] - adata.shape[0]
+        if verbose:
+            if adata.shape[0] != orig_shape[0]:
+                print(
+                    "Ignoring {} cells with less than {} genes detected".format(
+                        orig_shape[0] - adata.shape[0], min_genes
+                    )
                 )
-            )
-        if adata.shape[1] != orig_shape[1]:
-            print(
-                "Ignoring {} genes with zero total counts".format(
-                    orig_shape[1] - adata.shape[1]
+            if adata.shape[1] != orig_shape[1]:
+                print(
+                    "Ignoring {} genes with zero total counts".format(
+                        orig_shape[1] - adata.shape[1]
+                    )
                 )
-            )
+        adata.obs.drop(columns=["n_genes"], inplace=True)
+        adata.var.drop(columns=["n_counts"], inplace=True)
 
     # store raw counts before manipulation
     adata.layers["raw_counts"] = adata.X.copy()
 
     if calc_metrics:
-        if verbose:
-            print("Calculating metrics:")
         # identify mitochondrial genes
         adata.var["mito"] = adata.var_names.str.contains(mito_names)
         # identify putative ambient genes by lowest dropout pct (top n_ambient)
-        adata.var["ambient"] = np.array(
-            adata.X.astype(bool).sum(axis=0) / adata.n_obs
+        adata.var["pct_dropout_by_counts"] = np.array(
+            (1 - (adata.X.astype(bool).sum(axis=0) / adata.n_obs)) * 100
         ).squeeze()
         lowest_dropout = round(
-            (1 - adata.var.ambient.nlargest(n=n_ambient).max()) * 100, 3
+            adata.var.pct_dropout_by_counts.nsmallest(n=n_ambient).min(), 3
         )
         highest_dropout = round(
-            (1 - adata.var.ambient.nlargest(n=n_ambient).min()) * 100, 3
-        )
-        adata.var["ambient"] = (
-            adata.var.ambient >= adata.var.ambient.nlargest(n=n_ambient).min()
+            adata.var.pct_dropout_by_counts.nsmallest(n=n_ambient).max(), 3
         )
+        adata.var["ambient"] = adata.var.pct_dropout_by_counts <= highest_dropout
+        # reorder genes by dropout rate
+        adata = adata[:, np.argsort(adata.var.pct_dropout_by_counts)].copy()
         if verbose:
             print(
                 "Top {} ambient genes have dropout rates between {} and {} percent:\n\t{}".format(
@@ -107,52 +112,86 @@ def recipe_dropkick(
         sc.pp.calculate_qc_metrics(
             adata, qc_vars=["mito", "ambient"], inplace=True, percent_top=None
         )
+        # remove pesky unneeded columns from .obs and .var
+        adata.obs.drop(
+            columns=adata.obs.columns[adata.obs.columns.str.startswith("log1p_")].union(
+                adata.obs.columns[adata.obs.columns.str.contains("total_counts_")]
+            ),
+            inplace=True,
+        )
+        adata.var.drop(
+            columns=adata.var.columns[adata.var.columns.str.startswith("log1p_")],
+            inplace=True,
+        )
         # other arcsinh-transformed metrics
         adata.obs["arcsinh_total_counts"] = np.arcsinh(adata.obs["total_counts"])
         adata.obs["arcsinh_n_genes_by_counts"] = np.arcsinh(
             adata.obs["n_genes_by_counts"]
         )
+        # make sure everything went well...
+        np.testing.assert_approx_equal(
+            adata.var.pct_dropout_by_counts.min(),
+            lowest_dropout,
+            significant=7,
+            verbose=True,
+        )
 
-    # log1p transform (adata.layers["log1p_norm"])
+    # normalize counts before transforming
     sc.pp.normalize_total(adata, target_sum=target_sum, layers=None, layer_norm=None)
-    adata.layers["norm_counts"] = adata.X.copy()  # save to .layers
-    sc.pp.log1p(adata)
-    adata.layers["log1p_norm"] = adata.X.copy()  # save to .layers
+    adata.layers["norm_counts"] = adata.X.copy()
 
     # HVGs
     if n_hvgs is not None:
         if verbose:
             print("Determining {} highly variable genes".format(n_hvgs))
+        # log1p transform for HVGs (adata.layers["log1p_norm"])
+        sc.pp.log1p(adata)
+        adata.layers["log1p_norm"] = adata.X.copy()  # save to .layers
         sc.pp.highly_variable_genes(
             adata, n_top_genes=n_hvgs, n_bins=20, flavor="seurat"
         )
+        adata.var.drop(columns=["dispersions", "dispersions_norm"], inplace=True)
 
-    # arcsinh-transform normalized counts to leave in .X
+    # arcsinh-transform normalized counts (adata.layers["arcsinh_norm"])
     adata.X = np.arcsinh(adata.layers["norm_counts"])
     sc.pp.scale(adata)  # scale genes for feeding into model
     adata.layers[
         "arcsinh_norm"
     ] = adata.X.copy()  # save arcsinh scaled counts in .layers
+    # remove unneeded stuff
+    del adata.layers["norm_counts"]
 
     # set .X as desired for downstream processing; default raw_counts
+    if (X_final != "raw_counts") & verbose:
+        print("Setting {} layer to .X".format(X_final))
     adata.X = adata.layers[X_final].copy()
 
+    return adata
+
 
 def auto_thresh_obs(
-    adata, obs_cols=["arcsinh_n_genes_by_counts", "pct_counts_ambient"], method="otsu",
+    adata,
+    obs_cols=["arcsinh_n_genes_by_counts", "pct_counts_ambient"],
+    # double=["arcsinh_n_genes_by_counts"],
+    # single=["pct_counts_ambient"],
+    method="otsu",
 ):
     """
     automated thresholding on metrics in adata.obs
 
     Parameters:
         adata (anndata.AnnData): object containing unfiltered scRNA-seq data
-        obs_cols (list of str): name of column(s) to threshold from adata.obs
+        double (list of str): name of column(s) to double threshold from adata.obs
+        single (list of str): name of column(s) to single threshold from adata.obs
         method (str): one of 'otsu' (default), 'li', or 'mean'
 
     Returns:
         thresholds (dict): keys are obs_cols and values are threshold results
     """
     thresholds = dict.fromkeys(obs_cols)  # initiate output dictionary
+    # for col in double:
+    #    tmp = np.array(adata.obs[col])
+    #    thresholds[col] = threshold_multiotsu(tmp)
     for col in obs_cols:
         tmp = np.array(adata.obs[col])
         if method == "otsu":
@@ -188,7 +227,10 @@ def plot_thresh_obs(adata, thresholds, bins=40, show=True):
     axes[0].set_ylabel("cells")
     for i in range(len(thresholds)):
         axes[i].hist(adata.obs[list(thresholds.keys())[i]], bins=bins)
-        axes[i].axvline(list(thresholds.values())[i], color="r")
+        if isinstance(list(thresholds.values())[i], np.ndarray):
+            [axes[i].axvline(_x, color="r") for _x in list(thresholds.values())[i]]
+        else:
+            axes[i].axvline(list(thresholds.values())[i], color="r")
         axes[i].set_title(list(thresholds.keys())[i])
     fig.tight_layout()
     if show:
@@ -253,15 +295,14 @@ def filter_thresh_obs(
 
 def dropkick(
     adata,
+    min_genes=50,
     mito_names="^mt-|^MT-",
+    n_ambient=10,
     n_hvgs=2000,
     thresh_method="otsu",
     metrics=["arcsinh_n_genes_by_counts", "pct_counts_ambient",],
     directions=["above", "below"],
     alphas=[0.1],
-    n_lambda=100,
-    cut_point=1,
-    n_splits=5,
     max_iter=1000,
     n_jobs=-1,
     seed=18,
@@ -273,8 +314,11 @@ def dropkick(
     Parameters:
         adata (anndata.AnnData): object containing unfiltered, raw scRNA-seq
             counts in .X layer
+        min_genes (int): threshold for minimum genes detected. Default 50.
+            Ignores all cells with less than min_genes (dropkick label = 0).
         mito_names (str): substring encompassing mitochondrial gene names for
             calculation of mito expression
+        n_ambient (int): number of ambient genes to call. top genes by cells.
         n_hvgs (int or None): number of HVGs to calculate using Seurat method
             if None, do not calculate HVGs
         thresh_method (str): one of 'otsu' (default), 'li', or 'mean'
@@ -283,11 +327,6 @@ def dropkick(
             direction to keep (label=1)
         alphas (tuple of int): alpha values to test using glmnet with n-fold
             cross validation
-        n_lambda (int): number of lambda values to test in glmnet
-        cut_point (float): The cut point to use for selecting lambda_best.
-            arg_max lambda
-            cv_score(lambda)>=cv_score(lambda_max)-cut_point*standard_error(lambda_max)
-        n_splits (int): number of splits for n-fold cross validation
         max_iter (int): number of iterations for glmnet optimization
         n_jobs (int): number of threads for cross validation by glmnet
         seed (int): random state for cross validation by glmnet
@@ -297,20 +336,22 @@ def dropkick(
         adata_thresh (dict): dictionary of automated thresholds on heuristics
         rc (LogisticRegression): trained logistic regression classifier
 
-        updated adata inplace to include 'train', 'dropkick_score', and
+        updates adata inplace to include 'train', 'dropkick_score', and
             'dropkick_label' columns in .obs
     """
     # 0) preprocess counts and calculate required QC metrics
     a = adata.copy()  # make copy of anndata before manipulating
-    recipe_dropkick(
+    a = recipe_dropkick(
         a,
-        X_final="arcsinh_norm",
         filter=True,
+        min_genes=min_genes,
         calc_metrics=True,
         mito_names=mito_names,
-        n_hvgs=n_hvgs,
+        n_ambient=n_ambient,
         target_sum=None,
-        verbose=True,
+        n_hvgs=n_hvgs,
+        X_final="arcsinh_norm",
+        verbose=verbose,
     )
 
     # 1) threshold chosen heuristics using automated method
@@ -327,7 +368,7 @@ def dropkick(
         name="train",
     )
 
-    X = a.X[:, a.var.highly_variable].copy()  # final X is HVGs
+    X = a[:, a.var.highly_variable].X.copy()  # X for testing
     y = a.obs["train"].copy(deep=True)  # final y is "train" labels from step 2
     print("Training LogitNet with alphas: {}".format(alphas))
 
@@ -338,9 +379,9 @@ def dropkick(
             # print("Training LogitNet with alpha: {}".format(alpha), end="  ")
             rc = LogitNet(
                 alpha=alpha,
-                n_lambda=n_lambda,
-                cut_point=cut_point,
-                n_splits=n_splits,
+                n_lambda=100,
+                cut_point=1,
+                n_splits=5,
                 max_iter=max_iter,
                 n_jobs=n_jobs,
                 random_state=seed,
@@ -366,9 +407,9 @@ def dropkick(
         # 3.2) train model with single alpha value
         rc_ = LogitNet(
             alpha=alphas[0],
-            n_lambda=n_lambda,
-            cut_point=cut_point,
-            n_splits=n_splits,
+            n_lambda=100,
+            cut_point=1,
+            n_splits=5,
             max_iter=max_iter,
             n_jobs=n_jobs,
             random_state=seed,
@@ -387,13 +428,12 @@ def dropkick(
     for metric in metrics:
         adata.obs.loc[a.obs_names, metric] = a.obs[metric]
         adata.obs[metric].fillna(0, inplace=True)  # fill ignored cells with zeros
-    adata.var.loc[
-        a.var_names[a.var.highly_variable], "dropkick_coef"
-    ] = rc_.coef_.squeeze()
+    adata.var.loc[a.var.highly_variable, "dropkick_coef"] = rc_.coef_.squeeze()
 
     # 5) save model hyperparameters in .uns
     adata.uns["dropkick_thresholds"] = adata_thresh
     adata.uns["dropkick_args"] = {
+        "n_ambient": n_ambient,
         "n_hvgs": n_hvgs,
         "thresh_method": thresh_method,
         "metrics": metrics,
@@ -405,9 +445,6 @@ def dropkick(
         "coef_path": rc_.coef_path_.squeeze().T,
         "cv_mean_score": rc_.cv_mean_score_,
         "cv_standard_error": rc_.cv_standard_error_,
-        "n_lambda": n_lambda,
-        "cut_point": cut_point,
-        "n_splits": n_splits,
         "max_iter": max_iter,
         "seed": seed,
     }  # save command-line arguments to .uns for reference
@@ -459,13 +496,41 @@ def coef_plot(adata, show=True):
     # plot coefficient values versus log(lambda) on left y-axis
     ax.set_title("Dropkick Coefficients")
     ax.set_xlabel("Log (lambda)")
-    ax.set_ylabel("Coefficient Value", color="g")
+    ax.set_ylabel("Coefficient Value")
     ax.plot(
         np.log(adata.uns["dropkick_args"]["lambda_path"]),
         adata.uns["dropkick_args"]["coef_path"],
         alpha=0.5,
     )
-    ax.tick_params(axis="y", labelcolor="g")
+    # plot top three genes by coefficient value
+    [
+        ax.text(
+            x=np.log(adata.uns["dropkick_args"]["chosen_lambda"]),
+            y=adata.var.dropkick_coef.max() + 0.15 - 0.05 * x,
+            s=" "
+            + adata.var.loc[-adata.var.dropkick_coef.isna(), "dropkick_coef"]
+            .nlargest(3)
+            .index[x],
+            fontsize=9,
+            color="g",
+        )
+        for x in range(3)
+    ]
+    # plot bottom three genes by coefficient value
+    [
+        ax.text(
+            x=np.log(adata.uns["dropkick_args"]["chosen_lambda"]),
+            y=adata.var.dropkick_coef.min() - 0.20 + 0.05 * x,
+            s=" "
+            + adata.var.loc[-adata.var.dropkick_coef.isna(), "dropkick_coef"]
+            .nsmallest(3)
+            .index[x],
+            fontsize=9,
+            color="r",
+        )
+        for x in range(3)
+    ]
+
     # plot CV scores versus log(lambda) on right y-axis
     ax2 = ax.twinx()
     ax2.set_ylabel("CV Mean Score", color="b")
